# -*- coding: utf-8 -*-
# $Id: tests.py 228249 2010-12-05 01:02:42Z glenfant $
"""Testing collective.monkeypatcherpanel"""
import unittest

from Testing import ZopeTestCase
import Products.Five
from Products.Five import zcml
from Products.Five import fiveconfigure

import collective.monkeypatcher
import collective.monkeypatcherpanel

fiveconfigure.debug_mode = True # Don't swallow install exceptions
zcml.load_config('meta.zcml', Products.Five)
zcml.load_config('configure.zcml', Products.Five)
zcml.load_config('configure.zcml', collective.monkeypatcher)
zcml.load_config('configure.zcml', collective.monkeypatcherpanel)
fiveconfigure.debug_mode = False

ZopeTestCase.installPackage('collective.monkeypatcherpanel')

# Some test resources
class DummyClass(object):
    """Blah Blah"""

    def someMethod(self):
        """someMethod docstring"""

        return "original result"

def patchedSomeMethod(self):
    return "patched result"


class MonkeyPatcherPanelTestCase(ZopeTestCase.ZopeTestCase):
    """Base for test cases"""

    def afterSetUp(self):
        # Adding the control panel
        dummy = self.app.unrestrictedTraverse('@@add-monkeypatcherpanel')()
        return

    def test_haveControlPanel(self):
        """Our control panel is installed"""

        cp = self.app.Control_Panel
        self.failUnless('collective_monkeypatcherpanel' in cp.objectIds())
        return

    def test_eventFired(self):
        """Our stupid patch"""

        this_module = __name__
        patch_zcml = """
        <configure
           xmlns="http://namespaces.zope.org/zope"
           xmlns:monkey="http://namespaces.plone.org/monkey"
           i18n_domain="collective.doesntmatter">
           <monkey:patch
              description="What the patch does in short"
              class="%(this_module)s.DummyClass"
              original="someMethod"
              replacement="%(this_module)s.patchedSomeMethod"
              />
        </configure>""" % locals()
        zcml.load_string(patch_zcml)
        all_patches = collective.monkeypatcherpanel.controlpanel.all_patches

        # We got 1 monkey patch
        self.failUnlessEqual(len(all_patches), 1)

        # We got expected patch information
        patch = all_patches[0]
        self.failUnlessEqual(patch['description'], u'What the patch does in short')
        self.failUnlessEqual(patch['original'], '%(this_module)s.DummyClass.someMethod' % locals())
        self.failUnlessEqual(patch['replacement'], '%(this_module)s.patchedSomeMethod' % locals())
        return

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(MonkeyPatcherPanelTestCase))
    return suite




# -*- coding: utf-8 -*-
# $Id: __init__.py 228249 2010-12-05 01:02:42Z glenfant $
"""collective.monkeypatcherpanel"""

import zope.component
from collective.monkeypatcher.interfaces import IMonkeyPatchEvent
from controlpanel import monkeyPatchAdded

# We need to register the event handler as soon as possible.
# Doing this as usual in ZCML will be too late.

gsm = zope.component.getGlobalSiteManager()
gsm.registerHandler(monkeyPatchAdded, (IMonkeyPatchEvent,))

def initialize(context):
    """Zope 2 registration"""
    # Maybe a future usage
    return



# -*- coding: utf-8 -*-
# $Id: controlpanel.py 146677 2010-10-21 15:24:51Z glenfant $
"""ZMI control panel"""

import Acquisition
from OFS import SimpleItem
from App.version_txt import getZopeVersion
version = getZopeVersion()
if version > (2, 12):
    class Fake:
        pass
else:
    from App.ApplicationManager import Fake

from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from config import CONTROLPANEL_ID

all_patches = []

def monkeyPatchAdded(event):
    """See collective.monkeypatcher.interfaces.IMonkeyPatchEvent"""
    all_patches.append(event.patch_info)
    return


class ControlPanel(Fake, SimpleItem.Item, Acquisition.Implicit):
    """The Monkey patches control panel"""

    id = CONTROLPANEL_ID
    name = title = "Monkey Patches"
    manage_main = PageTemplateFile('zmi/controlpanel.pt', globals())

    manage_options=((
        {'label': 'Monkey Patches', 'action': 'manage_main'},
        ))

    def getId(self):
        return self.id

    def allPatches(self):
        return all_patches




# -*- coding: utf-8 -*-
# $Id: setup.py 251030 2012-09-20 10:01:21Z ajung $
from setuptools import setup, find_packages
import os

def read(*names):
    here = os.path.dirname(os.path.abspath(__file__))
    path = os.path.join(here, *names)
    return open(path, 'r').read().strip()

version = read('collective', 'monkeypatcherpanel', 'version.txt')

setup(
    name='collective.monkeypatcherpanel',
    version=version,
    description="A Zope 2 control panel that shows monkey patches applied by collective.monkeypatcher",
    long_description=(read("README.txt")
                      + "\n\n"
                      + read('docs', 'HISTORY.txt')),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Framework :: Zope2",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
    keywords='zope monkey patch panel',
    author='Gilles Lenfant',
    author_email='gilles.lenfant@gmail.com',
    url='http://pypi.python.org/pypi/collective.monkeypatcherpanel',
    license='GPL',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['collective'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        # -*- Extra requirements: -*-
        'collective.monkeypatcher'
        ],
    entry_points="""
    [z3c.autoinclude.plugin]
    target=plone
    """,
    )



# -*- coding: utf-8 -*-
# $Id: utils.py 228249 2010-12-05 01:02:42Z glenfant $
"""Misc utilities"""

from Products.Five import BrowserView
from config import CONTROLPANEL_ID
from controlpanel import ControlPanel

class Utils(BrowserView):

    def add(self):
        """Add the control panel
        """
        zcp = self.context.Control_Panel
        if CONTROLPANEL_ID not in zcp.objectIds():
            mpcp = ControlPanel()
            zcp._setObject(CONTROLPANEL_ID, mpcp)
        return u"Monkey pacher panel added. Click the \"Back\" button of your browser."

    def remove(self):
        """Removes the control panel
        """
        zcp = self.context.Control_Panel
        if CONTROLPANEL_ID not in zcp.objectIds():
            zcp._delObject(CONTROLPANEL_ID)
        return u"Monkey pacher panel removed. Click the \"Back\" button of your browser."
